<?php

namespace Drupal\telega_bot\Service;

use Drupal\Core\Entity\EntityInterface;
use Longman\TelegramBot\Telegram;
use Longman\TelegramBot\Exception\TelegramException;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Longman\TelegramBot\Request;
use Drupal\Component\Serialization\Json;
use Longman\TelegramBot\Entities\InlineKeyboard;

/**
 * Bot saver service.
 */
class Bot {

  /**
   * Telegram.
   *
   * @var \Longman\TelegramBot\Telegram
   */
  protected $telegram;

  /**
   * Creates a new Bot manager.
   */
  public function __construct() {
    $this->etype = \Drupal::entityTypeManager();
    $this->d = \Drupal::service('date.formatter');
    $this->df = DateTimeItemInterface::DATETIME_STORAGE_FORMAT;

    // Enable admin users
    // $telegram->enableAdmins($admin_users);
    // Here you can set some command specific parameters
    // e.g. Google geocode/timezone api key for /date command
    // $telegram->setCommandConfig('date', [
    // 'google_api_key' => 'your_google_api_key_here'
    // ]);
    // End Extra, see
    // https://github.com/php-telegram-bot/example-bot
    // https://github.com/php-telegram-bot/core //.
  }

  /**
   * Init a new Bot manager.
   */
  public function init(EntityInterface $entity) {
    $this->entity = $entity;
    $this->id = $entity->id();
    $this->key = $entity->token->value;
    $this->name = $entity->name->value;
    /** @var \Longman\TelegramBot\Telegram $telegram */
    try {
      $telegram = new Telegram($this->key, $this->name);
      $this->telegram = $telegram;
    }
    catch (TelegramException $e) {
      $error = $e->getMessage();
      \Drupal::logger('telega')->notice("init fail $error " . $this->id);
      return $error;
    }
    return $this;
  }

  /**
   * Check Valuer in array.
   */
  private function getVal($data, $key) {
    $result = FALSE;
    if (!empty($data[$key])) {
      $result = $data[$key];
    }
    return $result;
  }

  /**
   * Check Valuer in array.
   */
  private function getTitle($data, $type) {
    $result = FALSE;
    $bot = $this->name;
    $title = "";
    $n = [];
    if (!empty($data['first_name'])) {
      $n[] = $data['first_name'];
    }
    if (!empty($data['last_name'])) {
      $n[] = $data['last_name'];
    }
    if (!empty($data['username'])) {
      $n[] = "@{$data['username']}";
    }
    if (empty($n)) {
      $n[] = $data['id'];
    }
    $name = implode(" ", $n);
    if ($type == 'user') {
      $title = $name;
    }
    if ($type == 'chat') {
      if (!empty($data['title'])) {
        $title = $data['title'];
      }
      else {
        $title = "{$name} | @$bot";
      }
    }
    return $title;
  }

  /**
   * User.
   */
  private function user($message) {
    if (!empty($message['from'])) {
      $storage = $this->etype->getStorage('telega_user');
      $u = $message['from'];
      $user = $this->query($storage, $u['id']);
      if (!$user) {
        $user = $storage->create([
          'title' => $this->getTitle($u, 'user'),
          'key' => $u['id'],
          'bot' => $u['is_bot'],
          'name' => $this->getVal($u, 'username'),
          'first_name' => $this->getVal($u, 'first_name'),
          'last_name' => $this->getVal($u, 'last_name'),
          'lang' => $this->getVal($u, 'language_code'),
          // 'yaml' => Yaml::dump($u, 8),
          // 'data' => FALSE,
        ]);
        $user->save();
      }
      return $user;
    }
  }

  /**
   * Chat.
   */
  private function chat($message) {
    if (!empty($message['chat'])) {
      $c = $message['chat'];
      $storage = $this->etype->getStorage('telega_chat');
      $chat = $this->query($storage, $c['id'], $c['type'] == 'private');
      if (!$chat) {
        $chat = $storage->create([
          'title' => $this->getTitle($c, 'chat'),
          'key' => $c['id'],
          'type' => $c['type'],
          'name' => $this->getVal($c, 'username'),
          'first_name' => $this->getVal($c, 'first_name'),
          'last_name' => $this->getVal($c, 'last_name'),
          'bot_id' => $this->id,
          'all_admins' => FALSE,
          'key_old' => FALSE,
          // 'yaml' => Yaml::dump($c, 8),
          // 'data' => $c,
        ]);
        $chat->save(TRUE);
      }
      return $chat;
    }
  }

  /**
   * Query.
   */
  private function query($storage, $key, $bot = FALSE) {
    $entities = [];
    $query = $storage->getQuery()
      ->condition('status', 1)
      ->condition('key', $key)
      ->range(0, 1);
    if ($bot) {
      $query->condition('bot_id', $this->id);
    }
    $ids = $query->execute();
    if (!empty($ids)) {
      foreach ($storage->loadMultiple($ids) as $id => $entity) {
        $entities[$id] = $entity;
      }
    }
    return array_shift($entities);
  }

  /**
   * Send message.
   */
  public function send(string $chat, string $text) : array {
    $json = Request::sendMessage([
      'chat_id' => $chat,
      'text'    => $text,
    ]);
    $data = Json::decode($json);
    return $data;
  }

  /**
   * Send message.
   */
  public function sendInlineKeyboard(string $chat, string $text) : array {
    $kboard = [
      ['text' => '🔴', 'callback_data' => 'b1'],
      ['text' => '🔵 2', 'callback_data' => 'b2'],
      ['text' => '3 🟢', 'callback_data' => 'b3'],
    ];
    $kboard2 = [
      ['text' => '4', 'callback_data' => 'b4'],
      ['text' => '5 ⭕️', 'callback_data' => 'b5'],
      ['text' => '6', 'callback_data' => 'b6'],
    ];
    $kboard3 = [
      ['text' => '7', 'callback_data' => 'b7'],
      ['text' => '8', 'callback_data' => 'b8'],
      ['text' => '9', 'callback_data' => 'b9'],
    ];
    $inline_keyboard = new InlineKeyboard($kboard, $kboard2, $kboard3);
    $json = Request::sendMessage([
      'chat_id' => $chat,
      'text'    => $text,
      'reply_markup' => $inline_keyboard,
    ]);
    $data = Json::decode($json);
    return $data;
  }

  /**
   * Save Data.
   */
  public function saveData(EntityInterface $bot, array $data) {
    $this->init($bot);
    // \Drupal::logger('telega_bot')->warning(print_r($data, TRUE)); // Debug.
    if (!empty($data['message'])) {
      $m = $data['message'];
      $user = $this->user($m);
      $chat = $this->chat($m);
      $title = mb_substr($this->getVal($m, 'text'), 0, 40);
      $message = [
        'title' => !empty($title) ? $title : '—',
        'key' => $m['message_id'],
        'date' => $this->d->format($m['date'], 'custom', $this->df),
        'text' => $this->getVal($m, 'text'),
        'bot_id' => $this->id,
        'chat_id' => $chat->id(),
        'user_id' => $user->id(),
        'entities' => $this->getVal($m, 'entities'),
        // 'yaml' => Yaml::encode($data, 8),
      ];
      $this->saveMessageEntity($message);
    }
    elseif (!empty($data['callback_query']['message'])) {
      $m = $data['callback_query']['message'];
      $user = $this->user($m);
      $chat = $this->chat($m);
      $data = $data['callback_query']['data'];
      $message = [
        'title' => 'callback_query',
        'key' => $m['message_id'],
        'date' => $this->d->format($m['date'], 'custom', $this->df),
        'text' => $this->getVal($m, 'text') . " = $data",
        'bot_id' => $this->id,
        'chat_id' => $chat->id(),
        'user_id' => $user->id(),
        'entities' => $this->getVal($m, 'entities'),
        // 'yaml' => Yaml::encode($data, 8),
      ];
      $this->saveMessageEntity($message);
    }
    elseif (!empty($data['my_chat_member'])) {
      $m = $data['my_chat_member'];
      $user = $this->user($m);
      $chat = $this->chat($m);
      $new = $m['new_chat_member'];
      $title = "{$new['user']['username']} ({$new['status']})";
      $message = [
        'title' => "$title | chat_member",
        'key' => $data['update_id'],
        'date' => $this->d->format($m['date'], 'custom', $this->df),
        'text' => $new['status'],
        'bot_id' => $this->id,
        'chat_id' => $chat->id(),
        'user_id' => $user->id(),
        'entities' => $this->getVal($m, 'entities'),
        // 'yaml' => Yaml::encode($data, 8),
      ];
      $this->saveMessageEntity($message);
    }
    else {
      \Drupal::logger('telega_bot')->error("bot:" . $bot->id());
      \Drupal::logger('telega_bot')->error(print_r($data, TRUE));
    }
  }

  /**
   * Save Message entity.
   */
  private function saveMessageEntity(array $message) : bool {
    if (\Drupal::moduleHandler()->moduleExists('telega_message')) {
      $storage = $this->etype->getStorage('telega_message');
      $storage->create($message)->save();
      return TRUE;
    }
    else {
      // \Drupal::logger('telega_bot')->info("<pre>" . Yaml::encode($message) . "</pre>");
      return FALSE;
    }
  }

}
